@@ -1,7 +1,6 @@
 # pyre-unsafe
 import csv
 import json
-import logging
 import os
 import tempfile
 from collections import defaultdict
@@ -12,19 +11,14 @@ from dataclasses import (
     fields as dataclass_fields,
 )
 from datetime import datetime
-from threading import Lock
-from typing import Any, Callable, Dict, List, Tuple, Union
+from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 
 from ego4d.internal.credential_s3 import S3Helper
 
 from tqdm import tqdm
 
 
-logging.basicConfig(filename="example.log", level=logging.DEBUG)
-lock = Lock()
-
-
-def split_s3_path(s3_path: str) -> Tuple[str, str]:
+def split_s3_path(s3_path: str) -> Tuple[Optional[str], str]:
     """
     Splits a full s3_path of the form "s3://bucket/folder/.../file.ext"
     into a tuple of ("bucket", "folder/.../file.ext")
@@ -368,53 +362,54 @@ def load_dataclass_dict_from_csv(
 
 def load_standard_metadata_files(
     s3,
-    path: str,
     standard_metadata_folder: str,
 ) -> Tuple[Dict[str, Device], Dict[str, ComponentType], Dict[str, Scenario]]:
-    # Load consortium-wide metadata into memory
-    bucket, path = split_s3_path(path)
-    # uncomment the below line to download metadata to the specified metadata folder
-    # download_standard_metadata_files_from_s3(s3, bucket, path, standard_metadata_folder)
-    devices = load_dataclass_dict_from_csv(
-        f"{standard_metadata_folder}/device.csv", Device, "device_id"
-    )
-    component_types = load_dataclass_dict_from_csv(
-        f"{standard_metadata_folder}/component_type.csv",
-        ComponentType,
-        "component_type_id",
-    )
-    scenarios = load_dataclass_dict_from_csv(
-        f"{standard_metadata_folder}/scenario.csv", Scenario, "scenario_id"
-    )
+    bucket, path = split_s3_path(standard_metadata_folder)
+    s3_bucket = S3Helper(s3, bucket)
+    truncated, available_metadata_files = s3_bucket.ls(path)
+    assert not truncated, f"Folder {bucket}/{path} " "has too many entries"
+    available_files = {f.key.split("/")[-1] for f in available_metadata_files}
+
+    with tempfile.TemporaryDirectory("tmp_standard_metadata_folder") as tempdir:
+        # Load reqiured files
+        # Load device.csv
+        file_name = "device.csv"
+        assert file_name in available_files, (
+            f"required file {file_name} not found in " f"{bucket}/{path}"
+        )
+        local_file_path = f"{tempdir}/{file_name}"
+        s3_bucket.get_file(f"{path}/{file_name}", local_file_path)
+        devices = load_dataclass_dict_from_csv(
+            local_file_path,
+            Device,
+            "device_id",
+        )
+
+        file_name = "component_type.csv"
+        assert file_name in available_files, (
+            f"required file {file_name} not found in " f"{bucket}/{path}"
+        )
+        local_file_path = f"{tempdir}/{file_name}"
+        s3_bucket.get_file(f"{path}/{file_name}", local_file_path)
+        component_types = load_dataclass_dict_from_csv(
+            local_file_path,
+            ComponentType,
+            "component_type_id",
+        )
+
+        file_name = "scenario.csv"
+        assert file_name in available_files, (
+            f"required file {file_name} not found in " f"{bucket}/{path}"
+        )
+        local_file_path = f"{tempdir}/{file_name}"
+        s3_bucket.get_file(f"{path}/{file_name}", local_file_path)
+        scenarios = load_dataclass_dict_from_csv(
+            local_file_path, Scenario, "scenario_id"
+        )
     validate_standard_metadata_files(devices, component_types, scenarios)
     return devices, component_types, scenarios
 
 
-def download_standard_metadata_files_from_s3(
-    s3, bucket: str, path: str, standard_metadata_folder: str
-) -> Tuple[Dict[str, Device], Dict[str, ComponentType], Dict[str, Scenario]]:
-    s3_bucket = S3Helper(s3, bucket)
-    auxiliary_videos = s3_bucket.get_file(
-        os.path.join(path, "auxiliary_video_component_data_file.csv"),
-        os.path.join(
-            standard_metadata_folder + "auxiliary_video_component_data_file.csv"
-        ),
-    )
-    physical_settings = s3_bucket.get_file(
-        os.path.join(path, "physical_setting.csv"),
-        os.path.join(standard_metadata_folder + "physical_setting.csv"),
-    )
-    video_components = s3_bucket.get_file(
-        os.path.join(path, "video_component_file.csv"),
-        os.path.join(standard_metadata_folder + "video_component_file.csv"),
-    )
-    video_metadata = s3_bucket.get_file(
-        os.path.join(path, "video_metadata.csv"),
-        os.path.join(standard_metadata_folder + "video_metadata.csv"),
-    )
-    return auxiliary_videos, physical_settings, video_components, video_metadata
-
-
 def validate_standard_metadata_files(
     devices: Dict[str, Device],
     component_types: Dict[str, ComponentType],
