@@ -1,29 +1,31 @@
-import csv
-import logging
-import os
-import tempfile
-from pathlib import Path
-from typing import Any, Callable, Dict, List, Optional, Tuple, Union
-
-from ego4d.cli.manifest import list_videos_in_manifest, VideoMetadata
-from ego4d.cli.universities import BUCKET_TO_UNIV, UNIV_TO_BUCKET
-from ego4d.internal.ffmpeg_utils import get_video_info, VideoInfo
-from ego4d.internal.university_files import *
 import collections
-import logging
-import math
+import csv
+import functools
 from collections import defaultdict
 from concurrent.futures import ThreadPoolExecutor
-from threading import Lock
+from typing import Dict, List, Tuple
 
-import boto3
 import botocore
-import pandas as pd
 from botocore.exceptions import ClientError
-from ego4d.internal.credential_s3 import S3Helper
-from tqdm import tqdm
 
-lock = Lock()
+from ego4d.cli.manifest import VideoMetadata
+from ego4d.internal.ffmpeg_utils import get_video_info, VideoInfo
+from ego4d.internal.university_files import (
+    Annotations,
+    AuxiliaryVideoComponentDataFile,
+    ComponentType,
+    Device,
+    ErrorMessage,
+    load_standard_metadata_files,
+    load_university_files,
+    Particpant,
+    PhysicalSetting,
+    Scenario,
+    split_s3_path,
+    SynchronizedVideos,
+    VideoComponentFile,
+)
+from tqdm import tqdm
 
 """
 -i: The S3 path where the metadata is stored
@@ -34,12 +36,97 @@ lock = Lock()
 """
 
 
+def _validate_vcs(
+    video_id_component_pair: Tuple[str, List[VideoComponentFile]],
+    video_metadata_dict: Dict[str, VideoMetadata],
+    bucket_name: str,
+    s3,
+) -> Tuple[List[Tuple[str, str]], List[ErrorMessage]]:
+    errors = []
+    result = []
+
+    video_id, components = video_id_component_pair
+    components.sort(key=lambda x: x.component_index)
+
+    if video_id not in video_metadata_dict:
+        errors.append(
+            ErrorMessage(
+                video_id,
+                "video_not_found_in_video_metadata_error",
+                f"{video_id} in video_components_dict can't be found in video_metadata",
+            )
+        )
+        return result, errors
+    elif video_metadata_dict[video_id].number_video_components != len(components):
+        errors.append(
+            ErrorMessage(
+                video_id,
+                "video_component_length_inconsistent_error",
+                f"the video has {len(components)} components when it"
+                f" should have {video_metadata_dict[video_id].number_video_components}",
+            )
+        )
+    components.sort(key=lambda x: x.component_index)
+    # check component_index is incremental and starts at 0
+    university_video_folder_path = video_metadata_dict[
+        video_id
+    ].university_video_folder_path
+    for i in range(len(components)):
+        if components[i].component_index != i:
+            errors.append(
+                ErrorMessage(
+                    video_id,
+                    "video_component_wrong_index_error",
+                    f"the video component has index {components[i].component_index}"
+                    f" when it should have {i}",
+                )
+            )
+        if (
+            not components[i].video_component_relative_path
+            and not components[i].is_redacted
+        ):
+            errors.append(
+                ErrorMessage(
+                    video_id,
+                    "empty_video_component_relative_path_error",
+                    f"Found an empty relative path for the video_id {video_id}",
+                )
+            )
+        else:
+            if not components[i].is_redacted:
+                s3_path = f"{university_video_folder_path.rstrip('/')}/{components[i].video_component_relative_path}"
+                bucket, key = split_s3_path(s3_path)
+                if bucket != bucket_name:
+                    errors.append(
+                        ErrorMessage(
+                            video_id,
+                            "bucket_name_inconsistent_error",
+                            f"video has bucket_name {bucket_name}"
+                            f" when it should have {bucket}",
+                        )
+                    )
+                try:
+                    s3.head_object(Bucket=bucket, Key=key)
+                    result.append((video_id, key))
+                except ClientError:
+                    errors.append(
+                        ErrorMessage(
+                            video_id,
+                            "path_does_not_exist_error",
+                            f"video s3://{bucket}/{key} doesn't exist in bucket",
+                        )
+                    )
+
+    return result, errors
+
+
 def _validate_video_components(
     s3,
-    bucket_name,
+    bucket_name: str,
     video_metadata_dict: Dict[str, VideoMetadata],
     video_components_dict: Dict[str, List[VideoComponentFile]],
     error_message: List[ErrorMessage],
+    num_workers: int,
 ) -> List[Tuple[str, str]]:
     """
     Args:
@@ -55,81 +142,60 @@ def _validate_video_components(
         video component's university_video_id and key
     """
     print("validating video components")
+    map_fn = functools.partial(
+        _validate_vcs,
+        video_metadata_dict=video_metadata_dict,
+        bucket_name=bucket_name,
+        s3=s3,
+    )
     video_info_param = []
-    for video_id, components in tqdm(video_components_dict.items()):
-        if video_id not in video_metadata_dict:
-            error_message.append(
-                ErrorMessage(
-                    video_id,
-                    "video_not_found_in_video_metadata_error",
-                    f"{video_id} in video_components_dict can't be found in video_metadata",
-                )
-            )
-            continue
-        elif video_metadata_dict[video_id].number_video_components != len(components):
-            error_message.append(
-                ErrorMessage(
-                    video_id,
-                    "video_component_length_inconsistent_error",
-                    f"the video has {len(components)} components when it"
-                    f" should have {video_metadata_dict[video_id].number_video_components}",
-                )
-            )
-        components.sort(key=lambda x: x.component_index)
-        # check component_index is incremental and starts at 0
-        university_video_folder_path = video_metadata_dict[
-            video_id
-        ].university_video_folder_path
-        for i in range(len(components)):
-            if components[i].component_index != i:
-                error_message.append(
-                    ErrorMessage(
-                        video_id,
-                        "video_component_wrong_index_error",
-                        f"the video component has index {components[i].component_index}"
-                        f" when it should have {i}",
-                    )
-                )
-            if not components[i].video_component_relative_path:
-                error_message.append(
-                    ErrorMessage(
-                        video_id,
-                        "empty_video_component_relative_path_error",
-                        f"Found an empty relative path for the video_id {video_id}",
-                    )
-                )
-            else:
-                s3_path = f"{university_video_folder_path.rstrip('/')}/{components[i].video_component_relative_path}"
-                bucket, key = split_s3_path(s3_path)
-                if bucket != bucket_name:
-                    error_message.append(
-                        ErrorMessage(
-                            video_id,
-                            "bucket_name_inconsistent_error",
-                            f"video has bucket_name {bucket_name}"
-                            f" when it should have {bucket}",
-                        )
-                    )
-                try:
-                    s3.head_object(Bucket=bucket, Key=key)
-                    video_info_param.append((video_id, key))
-                except ClientError as e:
-                    error_message.append(
-                        ErrorMessage(
-                            video_id,
-                            "path_does_not_exist_error",
-                            f"video s3://{bucket}/{key} doesn't exist in bucket",
-                        )
-                    )
+    with ThreadPoolExecutor(num_workers) as pool:
+        vals_to_map = list(video_components_dict.items())
+        for res, errs in tqdm(pool.map(map_fn, vals_to_map), total=len(vals_to_map)):
+            video_info_param.extend(res)
+            error_message.extend(errs)
 
     return video_info_param
 
 
+def _get_presigned_url(
+    video_param: Tuple[str, str],
+    s3_client: botocore.client.BaseClient,
+    bucket_name: str,
+    expiration: int,
+):
+    errors = []
+    object_name = video_param[1]
+    try:
+        filename = s3_client.generate_presigned_url(
+            "get_object",
+            Params={"Bucket": bucket_name, "Key": object_name},
+            ExpiresIn=expiration,
+        )
+        return filename, errors
+    except ClientError as ex:
+        errors.append(
+            ErrorMessage(
+                video_param[0],
+                "video_does_not_exist_in_bucket_error",
+                f"video s3://{bucket_name}/{object_name} doesn't exist in bucket - {ex}",
+            )
+        )
+        return None, errors
+
+
+def _check_video(x):
+    video_info, errs = get_video_info(x[1], name=x[0][1])
+    return x[0], video_info, errs
+
+
 def _get_videos(
     s3,
     bucket_name: str,
     video_info_param: List[Tuple[str, str]],
     error_message: List[ErrorMessage],
+    num_workers: int,
+    expiry_time_sec: int,
 ) -> Dict[str, List[VideoInfo]]:
     """
     Args:
@@ -143,25 +209,36 @@ def _get_videos(
     """
     print("loading and validating videos")
     video_info_dict = defaultdict(list)
-    # check component mp4 exist in S3
-    def thread_helper(x):
-        video_info = get_video_info(s3, bucket_name, x, error_message)
-        if video_info != None:
-            lock.acquire()
+
+    print("Generating presigned urls", flush=True)
+    # NOTE: s3 is not thread safe, doing this just incase
+    fps = [
+        (
+            x,
+            _get_presigned_url(
+                video_param=x,
+                s3_client=s3,
+                bucket_name=bucket_name,
+                expiration=expiry_time_sec,
+            ),
+        )
+        for x in tqdm(video_info_param)
+    ]
+    for _, (_, errs) in fps:
+        if len(errs) > 0:
+            error_message.extend(errs)
+
+    fps = [(x, filename) for x, (filename, errs) in fps if len(errs) == 0]
+
+    print("Checking URLs", flush=True)
+    with ThreadPoolExecutor(max_workers=num_workers) as pool:
+        for x, video_info, errs in tqdm(
+            pool.map(_check_video, fps),
+            total=len(fps),
+        ):
             video_info_dict[x[0]].append(video_info)
-            lock.release()
+            error_message.extend(errs)
 
-    def run(thread_helper, video_info_param):
-        with ThreadPoolExecutor(max_workers=15) as pool:
-            results = list(
-                tqdm(
-                    pool.map(thread_helper, video_info_param),
-                    total=len(video_info_param),
-                )
-            )
-        return results
-
-    run(thread_helper, video_info_param)
     return video_info_dict
 
 
@@ -194,32 +271,32 @@ def _validate_mp4(
         total_vcodec = set()
         total_acodec = set()
         total_rotate = set()
-        total_dimension = set()
         total_size = set()
         total_vtb = set()
         total_sar = set()
         total_fps = set()
 
         for i, video_info in enumerate(video_infos):
-            # check consistent video codec
-            if video_info.vcodec != None:
-                total_vcodec.add(video_info.vcodec)
-
-            # check consistent audio time base
-            if video_info.acodec != None:
-                total_acodec.add(video_info.acodec)
-
-            # check consistent rotation
-            if video_info.rotate != None:
-                total_rotate.add(video_info.rotate)
-            else:
+            if video_info is None:
                 error_message.append(
                     ErrorMessage(
                         video_id,
-                        "rotation_info_not_found_warning",
-                        f"component {i} has no rotation information",
+                        "no_video_info_for_component",
+                        f"component {i} has no metadata",
                     )
                 )
+                continue
+
+            # check consistent video codec
+            if video_info.vcodec is not None:
+                total_vcodec.add(video_info.vcodec)
+
+            # check consistent audio time base
+            if video_info.acodec is not None:
+                total_acodec.add(video_info.acodec)
+
+            # check consistent rotation
+            total_rotate.add(video_info.rotate)
 
             # check consistent width and height
             if video_info.sample_width != None and video_info.sample_height != None:
@@ -258,7 +335,7 @@ def _validate_mp4(
                 total_fps.add(video_info.fps)
 
             # check null mp4 duration
-            if video_info.mp4_duration == None:
+            if video_info.mp4_duration is None:
                 error_message.append(
                     ErrorMessage(
                         video_id,
@@ -269,21 +346,38 @@ def _validate_mp4(
             else:
                 video_length = (
                     video_info.vstart + video_info.vduration
-                    if video_info.vstart != None
+                    if video_info.vstart is not None
                     else video_info.vduration
                 )
                 audio_length = (
                     video_info.astart + video_info.aduration
-                    if video_info.astart != None
+                    if video_info.astart is not None
                     else video_info.aduration
                 )
-                delta = abs(max(video_length, audio_length) - video_info.mp4_duration)
-                if delta > 0.5:
+                if video_length is None and audio_length is None:
+                    error_message.append(
+                        ErrorMessage(
+                            video_id,
+                            "no_video_or_audio_stream_duration",
+                            f"component {i} has no video or audio stream duration metadata",
+                        )
+                    )
+
+                if video_length is None:
+                    video_length = -1
+
+                if audio_length is None:
+                    audio_length = -1
+
+                video_stream_length = max(video_length, audio_length)
+                threshold = 2 / (video_info.fps or 30)
+                delta = abs(video_stream_length - video_info.mp4_duration)
+                if delta >= threshold:
                     error_message.append(
                         ErrorMessage(
                             video_id,
                             "mp4_duration_too_large_or_small_warning",
-                            f"component {i} has an mp4 duration that's too large or small",
+                            f"component {i}: mp4_duration={video_info.mp4_duration}, stream_duration={video_stream_length}, vsd={video_length}, asd={audio_length}",  # noqa
                         )
                     )
 
@@ -339,7 +433,7 @@ def _validate_mp4(
 
 def _validate_synchronized_videos(
     video_metadata_dict: Dict[str, VideoMetadata],
-    synchronized_video_dict: Dict[str, SynchronizedVideos],
+    synchronized_video_dict: Dict[str, List[SynchronizedVideos]],
     error_message: List[ErrorMessage],
 ) -> None:
     """
@@ -355,7 +449,7 @@ def _validate_synchronized_videos(
     if synchronized_video_dict:
         for video_grouping_id, components in synchronized_video_dict.items():
             for component in components:
-                for video_id, num in component.associated_videos:
+                for video_id, _ in component.associated_videos:
                     if video_id not in video_metadata_dict:
                         error_message.append(
                             ErrorMessage(
@@ -368,6 +462,7 @@ def _validate_synchronized_videos(
 
 def _validate_auxilliary_videos(
     video_metadata_dict: Dict[str, VideoMetadata],
+    video_components_dict: Dict[str, List[VideoComponentFile]],
     auxiliary_video_component_dict: Dict[str, List[AuxiliaryVideoComponentDataFile]],
     component_types: Dict[str, ComponentType],
     error_message: List[ErrorMessage],
@@ -385,7 +480,7 @@ def _validate_auxilliary_videos(
     # and that the component_type is valid
     print("validating auxiliary videos")
     if auxiliary_video_component_dict:
-        for video_id, components in auxiliary_video_component_dict.items():
+        for video_id, aux_components in auxiliary_video_component_dict.items():
             if video_id not in video_metadata_dict:
                 error_message.append(
                     ErrorMessage(
@@ -394,42 +489,54 @@ def _validate_auxilliary_videos(
                         f"{video_id} in auxiliary_video_component_dict can't be found in video_metadata",
                     )
                 )
-            elif video_metadata_dict[video_id].number_video_components != len(
-                components
-            ):
-                error_message.append(
-                    ErrorMessage(
-                        video_id,
-                        "video_component_length_inconsistent_error",
-                        f"the video has {len(components)} auxiliary components when it"
-                        f" should have {video_metadata_dict[video_id].number_video_components}",
-                    )
-                )
-            components.sort(key=lambda x: x.component_index)
-            for i in range(len(components)):
-                component = components[i]
-                if i != component.component_index:
+
+            else:
+                redacted_video_components = [
+                    component
+                    for component in video_components_dict[video_id]
+                    if component.is_redacted
+                ]
+                if video_metadata_dict[video_id].number_video_components - len(
+                    redacted_video_components
+                ) != len(aux_components):
                     error_message.append(
                         ErrorMessage(
                             video_id,
-                            "video_component_wrong_index_error",
-                            f"the video component has auxiliary component index {component.component_index}"
-                            f" when it should have {i}",
+                            "video_component_length_inconsistent_error",
+                            f"the video has {len(aux_components)} auxiliary components when it"
+                            f" should have {video_metadata_dict[video_id].number_video_components}",  # noqa
                         )
                     )
-                if component.component_type_id not in component_types:
-                    error_message.append(
-                        ErrorMessage(
-                            video_id,
-                            "component_type_id_not_found_error",
-                            f"auxiliary component's component_type_id: '{component.component_type_id} does not exist in component_types'",
+                non_redacted_video_components = [
+                    component.component_index
+                    for component in video_components_dict[video_id]
+                    if not component.is_redacted
+                ]
+                aux_components.sort(key=lambda x: x.component_index)
+                for i in range(len(aux_components)):
+                    component = aux_components[i]
+                    if non_redacted_video_components[i] != component.component_index:
+                        error_message.append(
+                            ErrorMessage(
+                                video_id,
+                                "video_component_wrong_index_error",
+                                f"the video component has auxiliary component index {component.component_index}"
+                                f" when it should have {non_redacted_video_components[i]}",  # noqa
+                            )
+                        )
+                    if component.component_type_id not in component_types:
+                        error_message.append(
+                            ErrorMessage(
+                                video_id,
+                                "component_type_id_not_found_error",
+                                f"auxiliary component's component_type_id: '{component.component_type_id} does not exist in component_types'",  # noqa
+                            )
                         )
-                    )
 
 
 def _validate_participant(
     video_metadata_dict: Dict[str, VideoMetadata],
-    participant_dict: Dict[str, Particpant],
+    participant_dict: Dict[str, List[Particpant]],
     error_message: List[ErrorMessage],
 ) -> None:
     """
@@ -442,14 +549,13 @@ def _validate_participant(
     """
     print("validating participants")
     if participant_dict:
-        for participant_id, components in participant_dict.items():
+        for participant_id, _ in participant_dict.items():
             if participant_id not in video_metadata_dict:
                 error_message.append(
                     ErrorMessage(
-                        video_id,
+                        participant_id,
                         "participant_not_found_error",
-                        f" participant '{participant_id}' not exist in "
-                        " video metadata",
+                        f"participant '{participant_id}' does not exist in video metadata",
                     )
                 )
 
@@ -603,38 +709,54 @@ def validate_university_files(  # noqa :C901
     s3: botocore.client.BaseClient,
     error_details_path: str,
     error_summary_path: str,
+    num_workers: int,
+    expiry_time_sec: int,
 ) -> List[ErrorMessage]:
     error_message = []
     # Check ids in video_components_dict are in video_metadata_dict
     # and the # of components is correct
-    s3_bucket = S3Helper(s3, bucket_name)
     video_info_param = []
 
     video_info_param = _validate_video_components(
-        s3, bucket_name, video_metadata_dict, video_components_dict, error_message
+        s3=s3,
+        bucket_name=bucket_name,
+        video_metadata_dict=video_metadata_dict,
+        video_components_dict=video_components_dict,
+        error_message=error_message,
+        num_workers=num_workers,
+    )
+    video_info_dict = _get_videos(
+        s3=s3,
+        bucket_name=bucket_name,
+        video_info_param=video_info_param,
+        error_message=error_message,
+        num_workers=num_workers,
+        expiry_time_sec=expiry_time_sec,
     )
-    video_info_dict = _get_videos(s3, bucket_name, video_info_param, error_message)
     _validate_mp4(video_info_dict, error_message)
     _validate_synchronized_videos(
-        video_metadata_dict, synchronized_video_dict, error_message
+        video_metadata_dict=video_metadata_dict,
+        synchronized_video_dict=synchronized_video_dict,
+        error_message=error_message,
     )
     _validate_auxilliary_videos(
-        video_metadata_dict,
-        auxiliary_video_component_dict,
-        component_types,
-        error_message,
+        video_metadata_dict=video_metadata_dict,
+        video_components_dict=video_components_dict,
+        auxiliary_video_component_dict=auxiliary_video_component_dict,
+        component_types=component_types,
+        error_message=error_message,
     )
     _validate_participant(video_metadata_dict, participant_dict, error_message)
     _validate_annotations(video_metadata_dict, annotations_dict, error_message)
     _validate_video_metadata(
-        video_metadata_dict,
-        component_types,
-        participant_dict,
-        scenarios,
-        devices,
-        physical_setting_dict,
-        video_components_dict,
-        error_message,
+        video_metadata_dict=video_metadata_dict,
+        video_components_dict=video_components_dict,
+        participant_dict=participant_dict,
+        scenarios=scenarios,
+        devices=devices,
+        physical_setting_dict=physical_setting_dict,
+        component_types=component_types,
+        error_message=error_message,
     )
 
     error_dict = collections.defaultdict(int)
@@ -658,11 +780,17 @@ def validate_university_files(  # noqa :C901
 
 
 def validate_all(
-    path, s3, standard_metadata_folder, error_details_path, error_summary_path
+    path,
+    s3,
+    standard_metadata_folder,
+    error_details_path,
+    error_summary_path,
+    num_workers,
+    expiry_time_sec,
 ):
     # get access to metadata_folder
     devices, component_types, scenarios = load_standard_metadata_files(
-        s3, path, standard_metadata_folder
+        s3, standard_metadata_folder
     )
     bucket, path = split_s3_path(path)
 
@@ -691,4 +819,6 @@ def validate_all(
         s3,
         error_details_path,
         error_summary_path,
+        num_workers,
+        expiry_time_sec=expiry_time_sec,
     )
